==> snowlog-1.0.3.tar.gz: snowlog-1.0.3/src/setup.c <==
/* Snowlog - An Apache logfile browser/analyzer.
 * 
 * Copyright 2004 Oliver Feiler <kiza@kcore.de>
 * http://snowlog.kcore.de/
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as   
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */
   
#include <ncurses.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>

#include "config.h"
#include "main.h"
#include "filters.h"
#include "snowtk.h"
#include "loadlog.h"
#include "regexp.h"

struct regexp ** useragents = NULL;
int useragents_count = 0;
struct regexp ** referers = NULL;
int referers_count = 0;

char *logfile = "/usr/local/apache2/logs/access_log.parsable.0";
char *browser;

void PrintInvalidParamError (char * param) {
	printf("Parameter format for \"%s\" invalid. See \"snowlog -h\".\n", param);
}

void ParseCmdl (int argc, char *argv[]) {
	int i;
	
	for (i = 1; i < argc; i += 2) {
		if ((strcmp(argv[i], "--help") == 0) || (strcmp(argv[i], "-h") == 0)) {
			printf("Snowlog version %s\n\n", VERSION);
			printf("usage: snowlog [-hf] [--help|--logfile]\n\n");
			printf("--help|-h\t\tPrint this help message.\n");
			printf("--logfile|-f file\tLogfile to load (Default is\n");
			printf("\t\t\t\"%s\")\n", logfile);
			printf("\n");
			exit(1);
		} else if ((strcmp(argv[i], "--logfile") == 0) || (strcmp(argv[i], "-f") == 0)) {
			if (argv[i+1] != NULL)
				logfile = strdup(argv[i+1]);
			else {
				PrintInvalidParamError(argv[i]);
				exit(2);
			}
		} else if ((strcmp(argv[i], "--version") == 0) || (strcmp(argv[i], "-V") == 0)) {
			printf("Snowlog version %s\n", VERSION);
			exit(1);
		} else {
			printf("Unknown option \"%s\" given.\n", argv[i]);
			exit(3);
		}
	}
}

char * GetUser (void) {
	return strdup(getenv("USER"));
}

char * GetHome (void) {
	return strdup(getenv("HOME"));
}

/* Load browser command from ~./snowlog/browser. */
void SetupBrowser (char * file) {
	char tmp[512];
	FILE *configfile;

	configfile = fopen (file, "r");
	if (configfile == NULL) {
		UIStatus ("Creating new config \"browser\"...", 0, 0, LINES-1);
		configfile = fopen (file, "w+");
		browser = strdup("lynx %s");
		if (configfile == NULL) {
			/* Ignore this error as it's not overly important. */
			return;
		}
		fputs (browser, configfile);
		fclose (configfile);
	} else {
		fgets (tmp, sizeof(tmp), configfile);
		if (tmp[strlen(tmp)-1] == '\n')
			tmp[strlen(tmp)-1] = '\0';
		browser = strdup(tmp);
	}
}

void CheckDirs (char * homedir) {
	char file[512];
	char tmp[512];
	struct stat dirtest;
	/* 
	 * Setup config directories.
	 */
	snprintf (file, sizeof(file), "%s/.snowlog", homedir);
	if ((stat (file, &dirtest)) == -1 ) {
		/* Create directory. */
		if (mkdir (file, S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH) != 0) {
			snprintf(tmp, sizeof(tmp), "Could not create directory ~/.snowlog: %s", strerror(errno));
			UIStatus(tmp, 2, 1, LINES-1);
		}
	} else {
		/* Something with the name .snownews exists, let's see what it is. */
		if ((dirtest.st_mode & S_IFDIR) != S_IFDIR) {
			snprintf(tmp, sizeof(tmp), "Could not create directory ~/.snowlog: %s", strerror(errno));
			UIStatus(tmp, 2, 1, LINES-1);
		}
	}
}

void Config (int argc, char *argv[]) {
	char file[256];
	char tmp[256];
	char *user;
	char *homedir;
	char globaldata[256];
//	char browsers_regexp[256];
//	char referers_regexp[256];
	
	snprintf (globaldata, sizeof(globaldata), "%s/snowlog", DATAPATH);
//	snprintf (browsers_regexp, sizeof(browsers_regexp), "%s/useragents.regexp", globaldata);
//	snprintf (referers_regexp, sizeof(referers_regexp), "%s/referers.regexp", globaldata);
	
	ParseCmdl(argc, argv);
	user = GetUser();
	homedir = GetHome();
	
	InitCurses();
	SnowStartColors();
	
	/* Set umask to 077 for all created files. */
	umask(63);
	
	CheckDirs(homedir);
	
	snprintf (file, sizeof(file), "%s/.snowlog/browser", homedir);
	SetupBrowser (file);
	
	/* Load global configs */
	snprintf(file, sizeof(file), "%s/useragents.regexp", globaldata);
	LoadRegexps(file, &useragents, &useragents_count);
	snprintf(file, sizeof(file), "%s/referers.regexp", globaldata);
	LoadRegexps(file, &referers, &referers_count);
	snprintf(file, sizeof(file), "%s/filters", globaldata);
	LoadFilters(file);
	
	/* Append customized configs */
	snprintf(file, sizeof(file), "%s/.snowlog/useragents.regexp", homedir);
	LoadRegexps(file, &useragents, &useragents_count);
	snprintf(file, sizeof(file), "%s/.snowlog/referers.regexp", homedir);
	LoadRegexps(file, &referers, &referers_count);
	snprintf(file, sizeof(file), "%s/.snowlog/filters", homedir);
	LoadFilters(file);
	
	if (LogLoad(logfile) == -1) {
		snprintf(tmp, sizeof(tmp), "Could not load logfile \"%s\": %s\n", logfile, strerror(errno));
		MainQuit(tmp);
	}
	
	free(user);
	free(homedir);
}

==> snowlog-1.1.1.tar.gz: snowlog-1.1.1/src/setup.c <==
/* Snowlog - An Apache logfile browser/analyzer.
 * 
 * Copyright 2004 Oliver Feiler <kiza@kcore.de>
 * http://snowlog.kcore.de/
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as   
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */
   
#include <ncurses.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>

#include "config.h"
#include "main.h"
#include "filters.h"
#include "snowtk.h"
#include "loadlog.h"
#include "regexp.h"

struct regexp ** useragents = NULL;
int useragents_count = 0;
struct regexp ** referers = NULL;
int referers_count = 0;

char *logfile;
char *browser;

void PrintInvalidParamError (char * param) {
	printf("Parameter format for \"%s\" invalid. See \"snowlog -h\".\n", param);
}

void ParseCmdl (int argc, char *argv[]) {
	int i;
	
	for (i = 1; i < argc; i += 2) {
		if ((strcmp(argv[i], "--help") == 0) || (strcmp(argv[i], "-h") == 0)) {
			printf("Snowlog version %s\n\n", VERSION);
			printf("usage: snowlog [-hf] [--help|--logfile]\n\n");
			printf("--help|-h\t\tPrint this help message.\n");
			printf("--logfile|-f file\tLogfile to load (Current default is\n");
			printf("\t\t\t\"%s\")\n", logfile);
			printf("\n");
			exit(1);
		} else if ((strcmp(argv[i], "--logfile") == 0) || (strcmp(argv[i], "-f") == 0)) {
			if (argv[i+1] != NULL)
				logfile = strdup(argv[i+1]);
			else {
				PrintInvalidParamError(argv[i]);
				exit(2);
			}
		} else if ((strcmp(argv[i], "--version") == 0) || (strcmp(argv[i], "-V") == 0)) {
			printf("Snowlog version %s\n", VERSION);
			exit(1);
		} else {
			printf("Unknown option \"%s\" given.\n", argv[i]);
			exit(3);
		}
	}
}

char * GetUser (void) {
	return strdup(getenv("USER"));
}

char * GetHome (void) {
	return strdup(getenv("HOME"));
}

/* Load browser command from ~./snowlog/browser. */
void SetupBrowser (char * file) {
	char tmp[512];
	FILE *configfile;

	configfile = fopen (file, "r");
	if (configfile == NULL) {
		UIStatus ("Creating new config \"browser\"...", 0, 0, LINES-1);
		configfile = fopen (file, "w+");
		browser = strdup("lynx %s");
		if (configfile == NULL) {
			/* Ignore this error as it's not overly important. */
			return;
		}
		fputs (browser, configfile);
		fclose (configfile);
	} else {
		fgets (tmp, sizeof(tmp), configfile);
		if (tmp[strlen(tmp)-1] == '\n')
			tmp[strlen(tmp)-1] = '\0';
		browser = strdup(tmp);
	}
}

void CheckDirs (const char * homedir) {
	char file[512];
	char tmp[512];
	struct stat dirtest;
	/* 
	 * Setup config directories.
	 */
	snprintf (file, sizeof(file), "%s/.snowlog", homedir);
	if ((stat (file, &dirtest)) == -1 ) {
		/* Create directory. */
		if (mkdir (file, S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH) != 0) {
			snprintf(tmp, sizeof(tmp), "Could not create directory ~/.snowlog: %s", strerror(errno));
			UIStatus(tmp, 2, 1, LINES-1);
		}
	} else {
		/* Something with the name .snownews exists, let's see what it is. */
		if ((dirtest.st_mode & S_IFDIR) != S_IFDIR) {
			snprintf(tmp, sizeof(tmp), "Could not create directory ~/.snowlog: %s", strerror(errno));
			UIStatus(tmp, 2, 1, LINES-1);
		}
	}
}

/* Set the default logfiles.
   ~/.snowlog/default.log overrides system default
   Command line overrides user default. */
void setDefaultLog (const char * globaldata, const char * homedir) {
	char file[256];
	struct stat filetest;

	/* Set default logfile.
	   First we set it to /usr/local/share/snowlog/default.log */
	snprintf (file, sizeof(file), "%s/default.log", globaldata);
	logfile = strdup(file);

	/* Then, if it exists, we set it to ~/.snowlog/default.log */
	snprintf (file, sizeof(file), "%s/.snowlog/default.log", homedir);
	if ((stat (file, &filetest)) == 0)
		logfile = strdup(file);
}

void Config (int argc, char *argv[]) {
	char file[256];
	char tmp[256];
	char *user;
	char *homedir;
	char globaldata[256];

	snprintf (globaldata, sizeof(globaldata), "%s/snowlog", DATAPATH);
	
	user = GetUser();
	homedir = GetHome();

	setDefaultLog(globaldata, homedir);
	
	ParseCmdl(argc, argv);
	
	InitCurses();
	SnowStartColors();
	
	/* Set umask to 077 for all created files. */
	umask(63);
	
	CheckDirs(homedir);
	
	snprintf (file, sizeof(file), "%s/.snowlog/browser", homedir);
	SetupBrowser (file);
	
	/* Load global configs */
	snprintf(file, sizeof(file), "%s/useragents.regexp", globaldata);
	LoadRegexps(file, &useragents, &useragents_count);
	snprintf(file, sizeof(file), "%s/referers.regexp", globaldata);
	LoadRegexps(file, &referers, &referers_count);
	snprintf(file, sizeof(file), "%s/filters", globaldata);
	LoadFilters(file);
	
	/* Append customized configs */
	snprintf(file, sizeof(file), "%s/.snowlog/useragents.regexp", homedir);
	LoadRegexps(file, &useragents, &useragents_count);
	snprintf(file, sizeof(file), "%s/.snowlog/referers.regexp", homedir);
	LoadRegexps(file, &referers, &referers_count);
	snprintf(file, sizeof(file), "%s/.snowlog/filters", homedir);
	LoadFilters(file);
	
	if (LogLoad(logfile) == -1) {
		snprintf(tmp, sizeof(tmp), "Could not load logfile \"%s\": %s\n", logfile, strerror(errno));
		MainQuit(tmp);
	}
	
	free(user);
	free(homedir);
}
